package sun.util.calendar;

import java.util.zip.CRC32;
import java.io.BufferedInputStream;
import java.io.FileInputStream;
import java.io.File;
import java.security.PrivilegedAction;
import java.util.Locale;
import java.security.AccessController;
import sun.security.action.GetPropertyAction;
import java.util.HashMap;
import java.util.concurrent.ConcurrentHashMap;
import java.time.LocalDateTime;
import java.time.ZoneOffset;
import java.io.DataInput;
import java.io.IOException;
import java.io.StreamCorruptedException;
import java.util.Collections;
import java.io.DataInputStream;
import java.io.ByteArrayInputStream;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Map;

public final class ZoneInfoFile
{
    private static String versionId;
    private static final Map<String, ZoneInfo> zones;
    private static Map<String, String> aliases;
    private static byte[][] ruleArray;
    private static String[] regions;
    private static int[] indices;
    private static final boolean USE_OLDMAPPING;
    private static String[][] oldMappings;
    private static final long UTC1900 = -2208988800L;
    private static final long UTC2037 = 2145916799L;
    private static final long LDT2037 = 2114380800L;
    private static final long CURRT;
    static final int SECONDS_PER_DAY = 86400;
    static final int DAYS_PER_CYCLE = 146097;
    static final long DAYS_0000_TO_1970 = 719528L;
    private static final int[] toCalendarDOW;
    private static final int[] toSTZTime;
    private static final long OFFSET_MASK = 15L;
    private static final long DST_MASK = 240L;
    private static final int DST_NSHIFT = 4;
    private static final int TRANSITION_NSHIFT = 12;
    private static final int LASTYEAR = 2037;
    
    public static String[] getZoneIds() {
        int newLength = ZoneInfoFile.regions.length + ZoneInfoFile.oldMappings.length;
        if (!ZoneInfoFile.USE_OLDMAPPING) {
            newLength += 3;
        }
        String[] array = (String[])Arrays.copyOf(ZoneInfoFile.regions, newLength);
        int length = ZoneInfoFile.regions.length;
        if (!ZoneInfoFile.USE_OLDMAPPING) {
            array[length++] = "EST";
            array[length++] = "HST";
            array[length++] = "MST";
        }
        for (int i = 0; i < ZoneInfoFile.oldMappings.length; ++i) {
            array[length++] = ZoneInfoFile.oldMappings[i][0];
        }
        return array;
    }
    
    public static String[] getZoneIds(int n) {
        ArrayList<String> list = new ArrayList();
        for (String s : getZoneIds()) {
            if (getZoneInfo(s).getRawOffset() == n) {
                list.add(s);
            }
        }
        String[] a = (String[])list.toArray(new String[list.size()]);
        Arrays.sort(a);
        return a;
    }
    
    public static ZoneInfo getZoneInfo(String id) {
        if (id == null) {
            return null;
        }
        ZoneInfo zoneInfo0 = getZoneInfo0(id);
        if (zoneInfo0 != null) {
            zoneInfo0 = (ZoneInfo)zoneInfo0.clone();
            zoneInfo0.setID(id);
            return zoneInfo0;
        }
        return zoneInfo0;
    }
    
    private static ZoneInfo getZoneInfo0(String s) {
        try {
            ZoneInfo zoneInfo = (ZoneInfo)ZoneInfoFile.zones.get(s);
            if (zoneInfo != null) {
                return zoneInfo;
            }
            if (ZoneInfoFile.aliases.containsKey(s)) {
                s = (String)ZoneInfoFile.aliases.get(s);
            }
            int binarySearch = Arrays.binarySearch(ZoneInfoFile.regions, s);
            if (binarySearch < 0) {
                return null;
            }
            ZoneInfo zoneInfo2 = getZoneInfo(new DataInputStream(new ByteArrayInputStream(ZoneInfoFile.ruleArray[ZoneInfoFile.indices[binarySearch]])), s);
            ZoneInfoFile.zones.put(s, zoneInfo2);
            return zoneInfo2;
        }
        catch (Exception cause) {
            throw new RuntimeException("Invalid binary time-zone data: TZDB:" + s + ", version: " + ZoneInfoFile.versionId, cause);
        }
    }
    
    public static Map<String, String> getAliasMap() {
        return Collections.unmodifiableMap(ZoneInfoFile.aliases);
    }
    
    public static String getVersion() {
        return ZoneInfoFile.versionId;
    }
    
    public static ZoneInfo getCustomTimeZone(String s, int n) {
        return new ZoneInfo(toCustomID(n), n);
    }
    
    public static String toCustomID(int n) {
        int n2 = n / 60000;
        int n3;
        if (n2 >= 0) {
            n3 = '+';
        }
        else {
            n3 = '-';
            n2 = -n2;
        }
        int n4 = n2 / 60;
        int n5 = n2 % 60;
        char[] value = { 'G', 'M', 'T', n3, '0', '0', ':', '0', '0' };
        if (n4 >= 10) {
            int n6 = 4;
            value[n6] += n4 / 10;
        }
        int n7 = 5;
        value[n7] += n4 % 10;
        if (n5 != 0) {
            int n8 = 7;
            value[n8] += n5 / 10;
            int n9 = 8;
            value[n9] += n5 % 10;
        }
        return new String(value);
    }
    
    private ZoneInfoFile() {
    }
    
    private static void addOldMapping() {
        for (String[] array : ZoneInfoFile.oldMappings) {
            ZoneInfoFile.aliases.put(array[0], array[1]);
        }
        if (ZoneInfoFile.USE_OLDMAPPING) {
            ZoneInfoFile.aliases.put("EST", "America/New_York");
            ZoneInfoFile.aliases.put("MST", "America/Denver");
            ZoneInfoFile.aliases.put("HST", "Pacific/Honolulu");
        }
        else {
            ZoneInfoFile.zones.put("EST", new ZoneInfo("EST", -18000000));
            ZoneInfoFile.zones.put("MST", new ZoneInfo("MST", -25200000));
            ZoneInfoFile.zones.put("HST", new ZoneInfo("HST", -36000000));
        }
    }
    
    public static boolean useOldMapping() {
        return ZoneInfoFile.USE_OLDMAPPING;
    }
    
    private static void load(DataInputStream dataInputStream) throws ClassNotFoundException, IOException {
        if (dataInputStream.readByte() != 1) {
            throw new StreamCorruptedException("File format not recognised");
        }
        if (!"TZDB".equals(dataInputStream.readUTF())) {
            throw new StreamCorruptedException("File format not recognised");
        }
        int short1 = dataInputStream.readShort();
        for (int i = 0; i < short1; ++i) {
            ZoneInfoFile.versionId = dataInputStream.readUTF();
        }
        int short2 = dataInputStream.readShort();
        String[] array = new String[short2];
        for (int j = 0; j < short2; ++j) {
            array[j] = dataInputStream.readUTF();
        }
        int short3 = dataInputStream.readShort();
        ZoneInfoFile.ruleArray = new byte[short3][];
        byte[] b;
        for (int k = 0; k < short3; ZoneInfoFile.ruleArray[k] = b, ++k) {
            b = new byte[dataInputStream.readShort()];
            dataInputStream.readFully(b);
        }
        for (int l = 0; l < short1; ++l) {
            int short4 = dataInputStream.readShort();
            ZoneInfoFile.regions = new String[short4];
            ZoneInfoFile.indices = new int[short4];
            for (int n = 0; n < short4; ++n) {
                ZoneInfoFile.regions[n] = array[dataInputStream.readShort()];
                ZoneInfoFile.indices[n] = dataInputStream.readShort();
            }
        }
        ZoneInfoFile.zones.remove("ROC");
        for (int n2 = 0; n2 < short1; ++n2) {
            int short5 = dataInputStream.readShort();
            ZoneInfoFile.aliases.clear();
            for (int n3 = 0; n3 < short5; ++n3) {
                ZoneInfoFile.aliases.put(array[dataInputStream.readShort()], array[dataInputStream.readShort()]);
            }
        }
        addOldMapping();
        return;
    }
    
    public static ZoneInfo getZoneInfo(DataInput dataInput, String s) throws Exception {
        dataInput.readByte();
        int int1 = dataInput.readInt();
        long[] array = new long[int1];
        for (int i = 0; i < int1; ++i) {
            array[i] = readEpochSec(dataInput);
        }
        int[] array2 = new int[int1 + 1];
        for (int j = 0; j < array2.length; ++j) {
            array2[j] = readOffset(dataInput);
        }
        int int2 = dataInput.readInt();
        long[] array3 = new long[int2];
        for (int k = 0; k < int2; ++k) {
            array3[k] = readEpochSec(dataInput);
        }
        int[] array4 = new int[int2 + 1];
        for (int l = 0; l < array4.length; ++l) {
            array4[l] = readOffset(dataInput);
        }
        int byte1 = dataInput.readByte();
        ZoneInfoFile.ZoneOffsetTransitionRule[] array5 = new ZoneInfoFile.ZoneOffsetTransitionRule[byte1];
        for (int n = 0; n < byte1; ++n) {
            array5[n] = new ZoneOffsetTransitionRule(dataInput);
        }
        return getZoneInfo(s, array, array2, array3, array4, array5);
    }
    
    public static int readOffset(DataInput dataInput) throws IOException {
        int byte1 = dataInput.readByte();
        return (byte1 == 127) ? dataInput.readInt() : (byte1 * 900);
    }
    
    static long readEpochSec(DataInput dataInput) throws IOException {
        int n = dataInput.readByte() & 0xFF;
        if (n == 255) {
            return dataInput.readLong();
        }
        return (long)((n << 16) + ((dataInput.readByte() & 0xFF) << 8) + (dataInput.readByte() & 0xFF)) * 900L - 4575744000L;
    }
    
    private static ZoneInfo getZoneInfo(String s, long[] array, int[] array2, long[] array3, int[] array4, ZoneInfoFile.ZoneOffsetTransitionRule[] array5) {
        int n = 0;
        int n2 = 0;
        int[] array6 = null;
        boolean n3 = false;
        int n4;
        if (array.length > 0) {
            n4 = array2[array2.length - 1] * 1000;
            n3 = (array[array.length - 1] > ZoneInfoFile.CURRT);
        }
        else {
            n4 = array2[0] * 1000;
        }
        long[] array7 = null;
        int[] array8 = null;
        int newLength = 0;
        int newLength2 = 0;
        if (array3.length != 0) {
            array7 = new long[250];
            array8 = new int[100];
            int year = getYear(array3[array3.length - 1], array4[array3.length - 1]);
            int i = 0;
            int j = 1;
            while (i < array3.length && array3[i] < -2208988800L) {
                ++i;
            }
            if (i < array3.length) {
                if (i < array3.length) {
                    array8[0] = array2[array2.length - 1] * 1000;
                    newLength = 1;
                }
                newLength = addTrans(array7, newLength2++, array8, newLength, -2208988800L, array4[i], getStandardOffset(array, array2, -2208988800L));
            }
            while (i < array3.length) {
                long n5 = array3[i];
                if (n5 > 2145916799L) {
                    year = 2037;
                    break;
                }
                while (j < array.length) {
                    long n6 = array[j];
                    if (n6 >= -2208988800L) {
                        if (n6 > n5) {
                            break;
                        }
                        if (n6 < n5) {
                            if (newLength + 2 >= array8.length) {
                                array8 = Arrays.copyOf(array8, array8.length + 100);
                            }
                            if (newLength2 + 1 >= array7.length) {
                                array7 = Arrays.copyOf(array7, array7.length + 100);
                            }
                            newLength = addTrans(array7, newLength2++, array8, newLength, n6, array4[i], array2[j + 1]);
                        }
                    }
                    ++j;
                }
                if (newLength + 2 >= array8.length) {
                    array8 = Arrays.copyOf(array8, array8.length + 100);
                }
                if (newLength2 + 1 >= array7.length) {
                    array7 = Arrays.copyOf(array7, array7.length + 100);
                }
                newLength = addTrans(array7, newLength2++, array8, newLength, n5, array4[i + 1], getStandardOffset(array, array2, n5));
                ++i;
            }
            while (j < array.length) {
                long n7 = array[j];
                if (n7 >= -2208988800L) {
                    int index = indexOf(array8, 0, newLength, array4[i]);
                    if (index == newLength) {
                        ++newLength;
                    }
                    array7[newLength2++] = (n7 * 1000L << 12 | ((long)index & 0xFL));
                }
                ++j;
            }
            if (array5.length > 1) {
                while (year++ < 2037) {
                    ZoneInfoFile.ZoneOffsetTransitionRule zoneOffsetTransitionRule;
                    long transitionEpochSecond;
                    for (int length2 = array5.length, k = 0; k < length2; newLength = addTrans(array7, newLength2++, array8, newLength, transitionEpochSecond, zoneOffsetTransitionRule.offsetAfter, zoneOffsetTransitionRule.standardOffset), ++k) {
                        zoneOffsetTransitionRule = array5[k];
                        transitionEpochSecond = zoneOffsetTransitionRule.getTransitionEpochSecond(year);
                        if (newLength + 2 >= array8.length) {
                            array8 = Arrays.copyOf(array8, array8.length + 100);
                        }
                        if (newLength2 + 1 >= array7.length) {
                            array7 = Arrays.copyOf(array7, array7.length + 100);
                        }
                    }
                }
                ZoneInfoFile.ZoneOffsetTransitionRule zoneOffsetTransitionRule2 = array5[array5.length - 2];
                ZoneInfoFile.ZoneOffsetTransitionRule zoneOffsetTransitionRule3 = array5[array5.length - 1];
                array6 = new int[10];
                if (zoneOffsetTransitionRule2.offsetAfter - zoneOffsetTransitionRule2.offsetBefore < 0 && zoneOffsetTransitionRule3.offsetAfter - zoneOffsetTransitionRule3.offsetBefore > 0) {
                    zoneOffsetTransitionRule2 = (zoneOffsetTransitionRule3 = zoneOffsetTransitionRule3);
                }
                array6[0] = zoneOffsetTransitionRule2.month - 1;
                int access$500 = zoneOffsetTransitionRule2.dom;
                int access$501 = zoneOffsetTransitionRule2.dow;
                if (access$501 == -1) {
                    array6[1] = access$500;
                    array6[2] = 0;
                }
                else if (access$500 < 0 || access$500 >= 24) {
                    array6[1] = -1;
                    array6[2] = ZoneInfoFile.toCalendarDOW[access$501];
                }
                else {
                    array6[1] = access$500;
                    array6[2] = -ZoneInfoFile.toCalendarDOW[access$501];
                }
                array6[3] = zoneOffsetTransitionRule2.secondOfDay * 1000;
                array6[4] = ZoneInfoFile.toSTZTime[zoneOffsetTransitionRule2.timeDefinition];
                array6[5] = zoneOffsetTransitionRule3.month - 1;
                int access$502 = zoneOffsetTransitionRule3.dom;
                int access$503 = zoneOffsetTransitionRule3.dow;
                if (access$503 == -1) {
                    array6[6] = access$502;
                    array6[7] = 0;
                }
                else if (access$502 < 0 || access$502 >= 24) {
                    array6[6] = -1;
                    array6[7] = ZoneInfoFile.toCalendarDOW[access$503];
                }
                else {
                    array6[6] = access$502;
                    array6[7] = -ZoneInfoFile.toCalendarDOW[access$503];
                }
                array6[8] = zoneOffsetTransitionRule3.secondOfDay * 1000;
                array6[9] = ZoneInfoFile.toSTZTime[zoneOffsetTransitionRule3.timeDefinition];
                n = (zoneOffsetTransitionRule2.offsetAfter - zoneOffsetTransitionRule2.offsetBefore) * 1000;
                if (array6[2] == 6 && array6[3] == 0 && s.equals("Asia/Amman")) {
                    array6[2] = 5;
                    array6[3] = 86400000;
                }
                if (array6[2] == 7 && array6[3] == 0 && s.equals("Asia/Amman")) {
                    array6[2] = 6;
                    array6[3] = 86400000;
                }
                if (array6[7] == 6 && array6[8] == 0 && s.equals("Africa/Cairo")) {
                    array6[7] = 5;
                    array6[8] = 86400000;
                }
            }
            else if (newLength2 > 0) {
                if (year < 2037) {
                    long n14 = 2114380800L - (long)(n4 / 1000);
                    int index2 = indexOf(array8, 0, newLength, n4 / 1000);
                    if (index2 == newLength) {
                        ++newLength;
                    }
                    array7[newLength2++] = (n14 * 1000L << 12 | ((long)index2 & 0xFL));
                }
                else if (array3.length > 2) {
                    int length = array3.length;
                    long n8 = array3[length - 2];
                    int n9 = array4[length - 2 + 1];
                    int standardOffset = getStandardOffset(array, array2, n8);
                    long n10 = array3[length - 1];
                    int n11 = array4[length - 1 + 1];
                    int standardOffset2 = getStandardOffset(array, array2, n10);
                    if (n9 > standardOffset && n11 == standardOffset2) {
                        int n12 = array3.length - 2;
                        ZoneOffset ofTotalSeconds = ZoneOffset.ofTotalSeconds(array4[n12]);
                        ZoneOffset ofTotalSeconds2 = ZoneOffset.ofTotalSeconds(array4[n12 + 1]);
                        LocalDateTime localDateTime = LocalDateTime.ofEpochSecond(array3[n12], 0, ofTotalSeconds);
                        if (ofTotalSeconds2.getTotalSeconds() > ofTotalSeconds.getTotalSeconds()) {
                        }
                        else {
                            localDateTime = localDateTime.plusSeconds((long)(array4[n12 + 1] - array4[n12]));
                        }
                        int n13 = array3.length - 1;
                        ZoneOffset ofTotalSeconds3 = ZoneOffset.ofTotalSeconds(array4[n13]);
                        ZoneOffset ofTotalSeconds4 = ZoneOffset.ofTotalSeconds(array4[n13 + 1]);
                        LocalDateTime ofEpochSecond = LocalDateTime.ofEpochSecond(array3[n13], 0, ofTotalSeconds3);
                        LocalDateTime plusSeconds;
                        if (ofTotalSeconds4.getTotalSeconds() > ofTotalSeconds3.getTotalSeconds()) {
                            plusSeconds = ofEpochSecond.plusSeconds((long)(array4[n13 + 1] - array4[n13]));
                        }
                        else {
                            plusSeconds = ofEpochSecond;
                        }
                        array6 = new int[] { localDateTime.getMonthValue() - 1, localDateTime.getDayOfMonth(), 0, localDateTime.toLocalTime().toSecondOfDay() * 1000, 0, plusSeconds.getMonthValue() - 1, plusSeconds.getDayOfMonth(), 0, plusSeconds.toLocalTime().toSecondOfDay() * 1000, 0 };
                        n = (n9 - standardOffset) * 1000;
                    }
                }
            }
            if (array7 != null && array7.length != newLength2) {
                if (newLength2 == 0) {
                    array7 = null;
                }
                else {
                    array7 = Arrays.copyOf(array7, newLength2);
                }
            }
            if (array8 != null && array8.length != newLength) {
                if (newLength == 0) {
                    array8 = null;
                }
                else {
                    array8 = Arrays.copyOf(array8, newLength);
                }
            }
            if (array7 != null) {
                ZoneInfoFile.Checksum o = new ZoneInfoFile.Checksum();
                for (int l = 0; l < array7.length; ++l) {
                    long n15 = array7[l];
                    int n16 = (int)(n15 >>> 4 & 0xFL);
                    int n17 = (n16 == 0) ? 0 : array8[n16];
                    int n18 = (int)(n15 & 0xFL);
                    int n19 = array8[n18];
                    o.update((n15 >> 12) + (long)n18);
                    o.update(n18);
                    o.update((n16 == 0) ? -1 : n16);
                }
                n2 = (int)o.getValue();
            }
        }
        return new ZoneInfo(s, n4, n, n2, array7, array8, array6, (boolean)n3);
    }
    
    private static int getStandardOffset(long[] array, int[] array2, long n) {
        int n2;
        for (n2 = 0; n2 < array.length && n >= array[n2]; ++n2) {}
        return array2[n2];
    }
    
    private static int getYear(long n, int n2) {
        long n3 = Math.floorDiv(n + (long)n2, 86400L) + 719528L - 60L;
        long n4 = 0L;
        if (n3 < 0L) {
            long n5 = (n3 + 1L) / 146097L - 1L;
            n4 = n5 * 400L;
            n3 += -n5 * 146097L;
        }
        long n6 = (400L * n3 + 591L) / 146097L;
        long n7 = n3 - (365L * n6 + n6 / 4L - n6 / 100L + n6 / 400L);
        if (n7 < 0L) {
            --n6;
            n7 = n3 - (365L * n6 + n6 / 4L - n6 / 100L + n6 / 400L);
        }
        return (int)(n6 + n4 + (long)(((int)n7 * 5 + 2) / 153 / 10));
    }
    
    private static int indexOf(int[] array, int i, int n, int n2) {
        n2 *= 1000;
        while (i < n) {
            if (array[i] == n2) {
                return i;
            }
            ++i;
        }
        array[i] = n2;
        return i;
    }
    
    private static int addTrans(long[] array, int n, int[] array2, int n2, long n3, int n4, int n5) {
        int index = indexOf(array2, 0, n2, n4);
        if (index == n2) {
            ++n2;
        }
        int index2 = 0;
        if (n4 != n5) {
            index2 = indexOf(array2, 1, n2, n4 - n5);
            if (index2 == n2) {
                ++n2;
            }
        }
        array[n] = (n3 * 1000L << 12 | ((long)(index2 << 4) & 0xF0L) | ((long)index & 0xFL));
        return n2;
    }
    
    static void access$000(DataInputStream dataInputStream) throws ClassNotFoundException, IOException {
        load(dataInputStream);
    }
    
    static {
        zones = new ConcurrentHashMap();
        ZoneInfoFile.aliases = new HashMap();
        ZoneInfoFile.oldMappings = new String[][] { { "ACT", "Australia/Darwin" }, { "AET", "Australia/Sydney" }, { "AGT", "America/Argentina/Buenos_Aires" }, { "ART", "Africa/Cairo" }, { "AST", "America/Anchorage" }, { "BET", "America/Sao_Paulo" }, { "BST", "Asia/Dhaka" }, { "CAT", "Africa/Harare" }, { "CNT", "America/St_Johns" }, { "CST", "America/Chicago" }, { "CTT", "Asia/Shanghai" }, { "EAT", "Africa/Addis_Ababa" }, { "ECT", "Europe/Paris" }, { "IET", "America/Indiana/Indianapolis" }, { "IST", "Asia/Kolkata" }, { "JST", "Asia/Tokyo" }, { "MIT", "Pacific/Apia" }, { "NET", "Asia/Yerevan" }, { "NST", "Pacific/Auckland" }, { "PLT", "Asia/Karachi" }, { "PNT", "America/Phoenix" }, { "PRT", "America/Puerto_Rico" }, { "PST", "America/Los_Angeles" }, { "SST", "Pacific/Guadalcanal" }, { "VST", "Asia/Ho_Chi_Minh" } };
        String lowerCase = ((String)AccessController.doPrivileged(new GetPropertyAction("sun.timezone.ids.oldmapping", "false"))).toLowerCase(Locale.ROOT);
        USE_OLDMAPPING = (lowerCase.equals("yes") || lowerCase.equals("true"));
        AccessController.doPrivileged(new PrivilegedAction<Object>() {
            @Override
            public Object run() {
                try (final DataInputStream dataInputStream = new DataInputStream(new BufferedInputStream(new FileInputStream(new File(System.getProperty("java.home") + File.separator + "lib", "tzdb.dat"))))) {
                    ZoneInfoFile.access$000(dataInputStream);
                }
                catch (Exception cause) {
                    throw new Error(cause);
                }
                return null;
            }
        });
        CURRT = System.currentTimeMillis() / 1000L;
        toCalendarDOW = new int[] { -1, 2, 3, 4, 5, 6, 7, 1 };
        toSTZTime = new int[] { 2, 0, 1 };
    }
    
    private static class Checksum extends CRC32
    {
        @Override
        public void update(int n) {
            this.update(new byte[] { (byte)(n >>> 24), (byte)(n >>> 16), (byte)(n >>> 8), (byte)n });
        }
        
        void update(long n) {
            this.update(new byte[] { (byte)(int)(n >>> 56), (byte)(int)(n >>> 48), (byte)(int)(n >>> 40), (byte)(int)(n >>> 32), (byte)(int)(n >>> 24), (byte)(int)(n >>> 16), (byte)(int)(n >>> 8), (byte)(int)n });
        }
    }
    
    private static class ZoneOffsetTransitionRule
    {
        private final int month;
        private final byte dom;
        private final int dow;
        private final int secondOfDay;
        private final boolean timeEndOfDay;
        private final int timeDefinition;
        private final int standardOffset;
        private final int offsetBefore;
        private final int offsetAfter;
        
        ZoneOffsetTransitionRule(DataInput dataInput) throws IOException {
            int int1 = dataInput.readInt();
            int n = (int1 & 0x380000) >>> 19;
            int n2 = (int1 & 0x7C000) >>> 14;
            int n3 = (int1 & 0xFF0) >>> 4;
            int n4 = (int1 & 0xC) >>> 2;
            int n5 = int1 & 0x3;
            this.month = int1 >>> 28;
            this.dom = (byte)(((int1 & 0xFC00000) >>> 22) - 32);
            this.dow = ((n == 0) ? -1 : n);
            this.secondOfDay = ((n2 == 31) ? dataInput.readInt() : (n2 * 3600));
            this.timeEndOfDay = (n2 == 24);
            this.timeDefinition = (int1 & 0x3000) >>> 12;
            this.standardOffset = ((n3 == 255) ? dataInput.readInt() : ((n3 - 128) * 900));
            this.offsetBefore = ((n4 == 3) ? dataInput.readInt() : (this.standardOffset + n4 * 1800));
            this.offsetAfter = ((n5 == 3) ? dataInput.readInt() : (this.standardOffset + n5 * 1800));
        }
        
        long getTransitionEpochSecond(int n) {
            long n2;
            if (this.dom < 0) {
                n2 = toEpochDay(n, this.month, lengthOfMonth(n, this.month) + 1 + this.dom);
                if (this.dow != -1) {
                    n2 = previousOrSame(n2, this.dow);
                }
            }
            else {
                n2 = toEpochDay(n, this.month, (int)this.dom);
                if (this.dow != -1) {
                    n2 = nextOrSame(n2, this.dow);
                }
            }
            if (this.timeEndOfDay) {
                ++n2;
            }
            int n3 = 0;
            switch (this.timeDefinition) {
                case 0: {
                    n3 = 0;
                    break;
                }
                case 1: {
                    n3 = -this.offsetBefore;
                    break;
                }
                case 2: {
                    n3 = -this.standardOffset;
                    break;
                }
            }
            return n2 * 86400L + (long)this.secondOfDay + (long)n3;
        }
        
        static final boolean isLeapYear(int n) {
            return (n & 0x3) == 0x0 && (n % 100 != 0 || n % 400 == 0);
        }
        
        static final int lengthOfMonth(int n, int n2) {
            switch (n2) {
                case 2: {
                    return isLeapYear(n) ? 29 : 28;
                }
                case 4:
                case 6:
                case 9:
                case 11: {
                    return 30;
                }
                default: {
                    return 31;
                }
            }
        }
        
        static final long toEpochDay(int n, int n2, int n3) {
            long n4 = (long)n;
            long n5 = (long)n2;
            long n6 = 0L + 365L * n4;
            long n7;
            if (n4 >= 0L) {
                n7 = n6 + ((n4 + 3L) / 4L - (n4 + 99L) / 100L + (n4 + 399L) / 400L);
            }
            else {
                n7 = n6 - (n4 / -4L - n4 / -100L + n4 / -400L);
            }
            long n8 = n7 + (367L * n5 - 362L) / 12L + (long)(n3 - 1);
            if (n5 > 2L) {
                --n8;
                if (!isLeapYear(n)) {
                    --n8;
                }
            }
            return n8 - 719528L;
        }
        
        static final long previousOrSame(long n, int n2) {
            return adjust(n, n2, 1);
        }
        
        static final long nextOrSame(long n, int n2) {
            return adjust(n, n2, 0);
        }
        
        static final long adjust(long n, int n2, int n3) {
            int n4 = (int)Math.floorMod(n + 3L, 7L) + 1;
            if (n3 < 2 && n4 == n2) {
                return n;
            }
            if ((n3 & 0x1) == 0x0) {
                int n6 = n4 - n2;
                return n + (long)((n6 >= 0) ? (7 - n6) : (-n6));
            }
            int n5 = n2 - n4;
            return n - (long)((n5 >= 0) ? (7 - n5) : (-n5));
        }
    }
}
